classdef SinusoidSumModel < handle
    properties
        views = cell(1,0);
        sinusoids = cell(1,0);

        timeInterval;
        samplingFrequency;
    end
    
    methods
        function obj = SinusoidSumModel(timeInterval, samplingFrequency)
            obj.timeInterval = timeInterval;
            obj.samplingFrequency = samplingFrequency;
        end
        
        function addSinusoid(obj,sinusoid)
            obj.sinusoids{end+1} = sinusoid;
            
            [t, signal] = obj.generate();
            for i = 1 : length(obj.views)
                obj.views{i}.update(t, signal);
            end
        end
        
        function addView(obj, view)
            [t, signal] = obj.generate();
            view.update(t, signal);
            obj.views{end+1} = view;
        end
    end
    
    methods(Access = private)
        function [t, signal] = generate(obj)
            t = 0 : 1/obj.samplingFrequency : obj.timeInterval;

            signal = zeros(size(t));
            
            for i = 1 : length(obj.sinusoids)
                signal = signal + obj.sinusoids{i}.amplitude * sin(2 * pi * t * obj.sinusoids{i}.frequency);
            end
        end
    end
end
